import base64
import tempfile
from textwrap import dedent
from typing import Any, Dict, Optional

from bs4 import BeautifulSoup
from mkdocs.config.base import Config
from mkdocs.plugins import BasePlugin
from mkdocs.structure.files import Files
from mkdocs.structure.pages import Page

RENDER_SWITCH = "# mkdocs: render"
HIDECODE_SWITCH = "# mkdocs: hidecode"
HIDEOUTPUT_SWITCH = "# mkdocs: hideoutput"


def _rendered_image_to_dir(
    save_img_dir: str,
    render_code: str,
    global_namespace: Optional[Dict[str, Any]] = None,
    local_namespace: Optional[Dict[str, Any]] = None,
) -> bool:
    # snippet to save a figure
    clearplot_code = """
    import matplotlib.pyplot as plt
    """
    clearplot_code = dedent(clearplot_code)

    savefig_code = f"""
    plt.savefig("{save_img_dir}", bbox_inches='tight')
    """
    savefig_code = dedent(savefig_code)
    closefig_code = "plt.close()"

    # create namespace if not passed
    global_namespace = {} if global_namespace is None else global_namespace
    local_namespace = {} if local_namespace is None else local_namespace

    # render image to
    exec(clearplot_code, global_namespace, local_namespace)
    exec(render_code, global_namespace, local_namespace)

    is_empty: bool = eval(
        "not bool(plt.gcf().get_axes())", global_namespace, local_namespace
    )

    if not is_empty:
        exec(savefig_code, global_namespace, local_namespace)

    exec(closefig_code, global_namespace, local_namespace)

    return is_empty


class RenderPlugin(BasePlugin):
    """An `mkdocs` plugin.
    This plugin defines the following event hooks:
    - `on_page_content`
    Check the [Developing Plugins](https://www.mkdocs.org/user-guide/plugins/#developing-plugins) page of `mkdocs`
    for more information about its plugin system.
    """

    def on_page_content(
        self, html: str, page: Page, config: Config, files: Files
    ) -> str:
        """Renders the code cells with matplotlib.

        Search for code cells in the passed HTML string. If there is a code cell and it starts
        with the correct comment, execute it and paste the rendered image in an img tag.

        Args:
            html: Input Html
            page: Page Info
            config: Mkdocs Config
            files: File Info

        Returns:
            Html with rendered images added.
        """
        soup = BeautifulSoup(html, features="html.parser")

        # create namespace
        global_namespace: Dict[str, Any] = {}
        local_namespace: Dict[str, Any] = {}

        for code_tag in soup.find_all("code"):
            raw_code: str = code_tag.text
            code_lines = raw_code.splitlines()
            is_render = RENDER_SWITCH in code_lines
            is_hidecode = HIDECODE_SWITCH in code_lines
            is_hideoutput = HIDEOUTPUT_SWITCH in code_lines

            temp_file = tempfile.NamedTemporaryFile(suffix=".png").name

            # skip if not a multi line code cell
            if code_tag.parent.name != "pre":
                continue

            # only render if cell start with correct comment
            if is_render:
                is_empty = _rendered_image_to_dir(
                    temp_file, code_tag.text, global_namespace, local_namespace
                )

                # insert image tag
                if not is_hideoutput and not is_empty:
                    with open(temp_file, "rb") as f:
                        encoded = base64.b64encode(f.read()).decode("ascii")
                        img_tag = soup.new_tag(
                            "img",
                            src="data:image/png;base64,{}".format(encoded),
                            width="70%",
                        )
                        parent_code_tag = code_tag.parent
                        parent_code_tag.insert_after(img_tag)
                        img_tag.wrap(soup.new_tag("center"))

                        if is_hidecode:
                            parent_code_tag.decompose()

        return str(soup)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['mkdocs_matplotlib']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.7.1',
 'mkdocs-material>=8.1.9',
 'mkdocs>=1.1.1',
 'seaborn>=0.10.0']

entry_points = \
{'mkdocs.plugins': ['mkdocs_matplotlib = '
                    'mkdocs_matplotlib.plugin:RenderPlugin']}

setup_kwargs = {
    'name': 'mkdocs-matplotlib',
    'version': '0.8.4',
    'description': 'Live rendering of python code using matplotlib',
    'long_description': '# Mkdocs-Matplotlib\n\n[![PyPI version](https://badge.fury.io/py/mkdocs-matplotlib.svg)](https://badge.fury.io/py/mkdocs-matplotlib)\n[![Test](https://github.com/AnHo4ng/mkdocs-matplotlib/actions/workflows/test.yml/badge.svg)](https://github.com/AnHo4ng/mkdocs-matplotlib/actions/workflows/test.yml)\n[![Release Pipeline](https://github.com/AnHo4ng/mkdocs-matplotlib/actions/workflows/release.yml/badge.svg)](https://github.com/AnHo4ng/mkdocs-matplotlib/actions/workflows/release.yml)\n[![Code Quality](https://github.com/AnHo4ng/mkdocs-matplotlib/actions/workflows/conde_quality.yml/badge.svg)](https://github.com/AnHo4ng/mkdocs-matplotlib/actions/workflows/conde_quality.yml)\n[![Documentation Status](https://readthedocs.org/projects/mkdocs-matplotlib/badge/?version=latest)](https://mkdocs-matplotlib.readthedocs.io/en/latest/?badge=latest)\n[![Python version](https://img.shields.io/badge/python-3.8-blue.svg)](https://pypi.org/project/kedro/)\n[![License: MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://github.com/AnHo4ng/mkdocs-matplotlib/blob/master/LICENCE)\n![pre-commit](https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white)\n![Black](https://img.shields.io/badge/code%20style-black-000000.svg)\n\n**Mkdocs-Matplotlib** is a plugin for [mkdocs](https://www.mkdocs.org/) which allows you to automatically generate matplotlib figures and add them to your documentation.\nSimply write the code as markdown into your documention.\n\n![screenshot](docs/assets/screenshot.png)\n\n## Quick Start\n\nThis plugin can be installed with `pip`\n\n```shell\npip install mkdocs-matplotlib\n```\nTo enable this plugin for mkdocs you need to add the following lines to your `mkdocs.yml`.\n\n```yaml\nplugins:\n  - mkdocs_matplotlib\n```\n\nTo render a code cell using matplotlib you simply have to add the comment `# mkdocs: render` at the top of the cell.\n\n```python\n# mkdocs: render\nimport matplotlib.pyplot as plt\nimport numpy as np\n\nxpoints = np.array([1, 8])\nypoints = np.array([3, 10])\n\nplt.plot(xpoints, ypoints)\n```\n',
    'author': 'An Hoang',
    'author_email': 'an.hoang@statworx.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://mkdocs-matplotlib.readthedocs.io/en/latest/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.9,<4.0',
}


setup(**setup_kwargs)
